function resstruct = weightLikelihood(dataset, zoomtb, varargin)        
    %calculate zoomin likelihood first and permutation test later
    %(zoomwperm.m)
    
    para.detailmarker = '';
    para.markerset = 'impute_dist';
    para.consv = '';
    para.known_gene = {};
    
    para = assignpara(para, varargin{:});

    %load data
    datastruct = loadData(dataset);        
    [ncond] = length(datastruct.conds);
    %set up outliers for later corr calculation
    if isfield(datastruct, 'outliers')
        for condi = 1:ncond
            datastruct.phenotype(condi, datastruct.outliers{condi}) = NaN;
        end
    end
    
    %load zoomin table; exclude those without SNPs already
    if ischar(zoomtb)
        zoomtb = load(zoomtb);
        fds = fieldnames(zoomtb);
        zoomtb = zoomtb.(fds{1});
    end
    
    if length(zoomtb) ~= ncond, error('Inconst conditions and marker sets\n'); end

    %load markers
    
    s = load(para.detailmarker);
    fd = fieldnames(s);
    probemarker = s.(fd{1});    
    probestraini = strIndexQuery(probemarker.sample, datastruct.sample);
    if sum(probestraini==-1) > 0
        error('some strains are not in %s\n', para.detailmarker);
    end    
    probegenotype = probemarker.(para.markerset);
    genotype = probegenotype(:, probestraini);

    clear s
            
    %load conservation score and prior

    if ~isempty(para.consv)
        scstruct = load(para.consv);
        fd = fieldnames(scstruct);
        scstruct = scstruct.(fd{1});
        
        priorindex = 2;
        orfi = strIndexQuery(scstruct.orf, datastruct.geneid);
        prior = ones(length(datastruct.geneid),1) * min(scstruct.prior(:,priorindex));
        for i = 1:length(orfi)
            if orfi(i) ~= -1
                prior(i) = scstruct.prior(orfi(i), priorindex);
            end
        end
    else
        prior = ones(length(datastruct.geneid),1); %uniform prior
        priorindex = NaN;
    end
    
    
    
    t = cputime();    
    runcond = 1:ncond;
    res = cell(length(runcond),1);
    for i = runcond
        %struct.regi, beta, stat
        if isempty(zoomtb{i})
            continue
        end
        [uri nr uregions] = uniqueRegion(zoomtb{i}(:,4));
        regi = cell2mat(zoomtb{i}(:,1));
        growth = datastruct.phenotype(i,:);
        %growth(datastruct.outliers{i}) = NaN; %setup outliers; already set
        %up at Line 7

        sub = NaN(size(regi,1),3); %[L(D|L,E) L(E|L) L(L)]
        for uli = 1:nr
            rowi = find(uri==uli);            
            %choose the most correlated marker in the region as base genotype
            tmpcor = corr(growth', datastruct.genotype(uregions{uli},:)','rows','pairwise');
            [tmpcor tmpi] = max(abs(tmpcor));            
            [tmp sub(rowi,1) sub(rowi,2)] = ...
                regLikelihood(datastruct.expression(regi(rowi,1),:)', ...
                genotype(regi(rowi,2),:)', ...
                growth', ...
                datastruct.genotype(uregions{uli}(tmpi),:)');     
            sub(rowi,3) = log(prior(regi(rowi,1)));
        end
        res{i} = sub;                
    end
    resstruct.rawlike = res;
    likesum = zeros(1,3);
    for i = runcond
        if isempty(resstruct.rawlike{i}), continue; end
        %only consider those are not extremely insignificant
        %likesum = likesum + sum(resstruct.rawlike{i}(prevzoom{i}(:,4)<=0.05,:));        
        likesum = likesum + sum(resstruct.rawlike{i});
    end
    %get weighting; weight is a column vec
    [weight fval exitflag] = linprog(-likesum', ...
        [-1 1 1; 0 -1 1],[-0.7 -0.2], [1 1 1],[3],[0,0,0]);
    resstruct.weight = weight;
    resstruct.exitflag = exitflag;
    %calculate weighted likehood
    resstruct.wlike = cell(length(runcond),1);        
    for i = runcond
        if isempty(resstruct.rawlike{i}), continue; end        
        sub = NaN(size(resstruct.rawlike{i},1),9); %[gi li wl1 wl2 wl3 wlike wlikerank corr corrank]
        sub(:,1:2) = cell2mat(zoomtb{i}(:,1));
        sub(:,3) = resstruct.rawlike{i}(:,1) * weight(1);
        sub(:,4) = resstruct.rawlike{i}(:,2) * weight(2);
        sub(:,5) = resstruct.rawlike{i}(:,3) * weight(3);
        sub(:,6) = sum(sub(:,3:5),2);
        resstruct.wlike{i} = sub;        
    end
    %precompiled list
    known_gi = strIndexQuery(datastruct.genename, para.known_gene);      
    known_gi(known_gi == -1) = [];
    %fill in likerank
    resstruct.wlike = addRank(datastruct, resstruct.wlike, zoomtb, known_gi);
    resstruct.priorindex = priorindex;
    
    fprintf('time: %f\n',cputime()-t);    
end


function [uri nr uregions] = uniqueRegion(regions)
    %regions: cell
    n = length(regions);
    uri = ones(n,1);
    nr = 1;
    uregions = {regions{1}};
    for i = 2:n
        if ~isempty(intersect(regions{i},regions{i-1}))
            uri(i) = nr;
        else
            nr = nr + 1;
            uregions = [uregions; {regions{i}}];
            uri(i) = nr;
        end
    end
end

function [newguilty] = addRank(datastruct, guilty, zoomtb, known_gi)    
    %if known_gi = []; don't change likelihood
    %add rank and corr and rank of abs(corr) in the same region
    ncond = length(zoomtb);
    newguilty = guilty;
    for condi = 1:ncond        
        if ~isempty(zoomtb{condi})
            %fill in newguilty with corr
            newguilty{condi}(:,8) = corr(datastruct.phenotype(condi,:)', ...
                datastruct.expression(newguilty{condi}(:,1),:)', 'rows','pairwise');
            %merge region
            region = {};
            for i = 1:size(zoomtb{condi},1)
                found = false;
                for j = 1:size(region,1)
                    if ~isempty(intersect(region{j,1},zoomtb{condi}{i,3})) 
                        found = true;
                        region{j,1} = union(region{j,1},zoomtb{condi}{i,3}); %mindex
                        region{j,2} = [region{j,2} i]; %rowi
                        break;
                    end
                end
                if ~found
                    region{end+1,1} = zoomtb{condi}{i,3};
                    region{end,2} = i;
                end
            end
            for ri = 1:size(region,1)
                %for each region, rank like(:,3)                
                rowi = region{ri,2};
                [tmp j] = intersect(guilty{condi}(rowi,1), known_gi);
                if ~isempty(j)
                    like_cap = sum(guilty{condi}(rowi,4:5),2);
                    like_cap = sort(like_cap);
                    like_cap = like_cap(floor(length(like_cap)*0.95));
                    newguilty{condi}(rowi(j),6) = ...
                        newguilty{condi}(rowi(j),3) + ...
                        max(like_cap, sum(newguilty{condi}(rowi(j),4:5),2));
                end
                likerank = ranking(newguilty{condi}(rowi,6),'descend');
                newguilty{condi}(rowi,7) = likerank;
                corrank = ranking(abs(newguilty{condi}(rowi,8)),'descend');
                newguilty{condi}(rowi,9) = corrank;                
            end
        end
    end
end

function r = ranking(data, mode)    
    n = length(data);
    uvalue = unique(data);    
    
    tmp = sort(uvalue, mode);
    r = zeros(n,1);
    count = 1;
    for i = 1:length(tmp)
        datai = find(data==tmp(i));
        r(datai) = count;
        count = count + length(datai);
    end
end
